program AVR_KeyBoardAuto;

{$NOSHADOW}

Device = mega128;

Define_Fuses
  LockBits0      = [];
  FuseBits0      = [BODEN];
  FuseBits1      = [CKOPT, SPIEN, JTAGEN, OCDEN];
  FuseBits2      = [];
  ProgMode       = JTAG;

Import SysTick, SerPort, MatrixPort, BeepPort;

From System Import Pipes;
From MatrixPort import MatrixTimer;

Define
  ProcClock   = 16000000;        {Hertz}
  SysTick     = 10, Timer2;      {msec}
  StackSize   = $0030, iData;
  FrameSize   = $0040, iData;
  SerPort     = 19200, Stop2;    {Baud, StopBits|Parity}
  RxBuffer    = 8, iData;
  TxBuffer    = 8, iData;

  BeepPort    = PortG, 4;

  MatrixRow   = PortC, 4;        {use PortC, start with bit4}
  MatrixCol   = PinC, 0;         {use PinC,  start with bit0}
  MatrixType  = 4, 4;            {4 Rows at PortC, 4 Columns at PinC}
  Debounce    = 2;
  MatrixPipe  = 16, 500, 200;    // PipeLen, FirstRepeat, RepeatRate, in msecs


Implementation

{$IDATA}

{--------------------------------------------------------------}
{ Type Declarations }
type
  (* enumeration "Keys" imported with "MatrixPort" *)
  tKeyLookUp  = array[keys] of byte;


{--------------------------------------------------------------}
{ Const Declarations }
const
  KeyLookUp : tKeyLookUp = ($3F, $06, $5B, $4f,
                            $66, $6D, $7D, $07,
                            $7F, $6F, $77, $7C,
                            $58, $5E, $79, $71);

{--------------------------------------------------------------}
{ Var Declarations }
{$IDATA}
var
  key           : keys;

{--------------------------------------------------------------}
{ functions }
procedure InitPorts;
begin
  DDRA:= $FF;                             // all output as LED drivers
end;

{--------------------------------------------------------------}
{ Main Program }
{$IDATA}

begin
{
  if MatrixTimerArray[Key1] > 0 then
    // key is pressed
  endif;
  case MatrixTimerArray[Key1] of
    0     : // key released
          |
    1..10 : // key initially activated
          |
    lastMatrixKey:
          |
  else
    // do any repeat action
  endcase;
}
  InitPorts;

  EnableInts;
  Start_Processes;
  loop
    If PipeStat(MatrixKeyPipe) <> 0 then
      key:= PipeRecv(MatrixKeyPipe);
      BeepClick;
      PortA:= KeyLookUp[key];
      Write(SerOut, ByteToHex(byte(key)) +' ');
    endif;
    nop;
  endloop;

end AVR_KeyBoardAuto.


